/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package beaconconfig;

import java.awt.event.*;
import java.util.*;

class RemoteControls 
    extends Controls implements RemoteSyncListener
{
    /* states of the root BeaconConfigApp */
    private RemoteSyncEvent currentStates;
    private EventListener eventHandler;
    private boolean override;

    public RemoteControls(DrawingArea d, SolverPanel s, 
			  EventListener e, boolean override)
    {
	super(d, s);
	currentStates = new RemoteSyncEvent();
	currentStates.controlPanelStates = super.getStates();
	eventHandler = e;
	this.override = override;
    }

    private void updateRemote()
    {
	if(!override) {
	    currentStates = new RemoteSyncEvent();
	    currentStates.controlPanelStates = super.getStates();
	}
	if(eventHandler instanceof RemoteSyncListener) {
	    // calls handler in NetworkServer
	    ((RemoteSyncListener) eventHandler).remoteSyncEvent(currentStates);
	}
    }

    public void enableManualOverride(boolean v) 
    {
	super.enableManualOverride(v);
	updateRemote();
    }

    public void enableTrackingOptions(boolean v)
    {
	super.enableTrackingOptions(v);
	updateRemote();
    }

    public void actionPerformed(ActionEvent e) 
    {
	Util.dbgPrintln("RemoteControls::actionPerformed "+e.getActionCommand());
	if(!override) { // root app
	    super.actionPerformed(e);
	    currentStates = new RemoteSyncEvent();
	    currentStates.controlPanelStates = super.getStates();
	}
	else {
	    // remote app: restore old states, then wait for ack from
	    // root application
	    setCurrentStates(currentStates);
	}
	if(eventHandler instanceof ActionListener) {
	    // calls handler in NetworkClient
	    ((ActionListener) eventHandler).actionPerformed(e);
	}
	else if(eventHandler instanceof RemoteSyncListener) {
	    // calls handler in NetworkServer
	    ((RemoteSyncListener) eventHandler).remoteSyncEvent(currentStates);
	}
    }

    private void setCurrentStates(RemoteSyncEvent cs)
    {
	Util.dbgPrintln("RemoteControls::setCurrentStates");
	currentStates = cs;
	if(cs.controlPanelStates != null) {
	    Boolean manualOverrideSelected = (Boolean) cs.controlPanelStates.get(0);
	    Boolean manualOverrideEnabled  = (Boolean) cs.controlPanelStates.get(1);
	    Boolean aggressiveSelected     = (Boolean) cs.controlPanelStates.get(2);
	    Boolean aggressiveEnabled      = (Boolean) cs.controlPanelStates.get(3);
	    Boolean gridSelected           = (Boolean) cs.controlPanelStates.get(4);
	    Boolean gridEnabled            = (Boolean) cs.controlPanelStates.get(5);
	    Boolean annulusSelected        = (Boolean) cs.controlPanelStates.get(6);
	    Boolean annulusEnabled         = (Boolean) cs.controlPanelStates.get(7);
	    Boolean streamerSelected       = (Boolean) cs.controlPanelStates.get(8);
	    Boolean streamerEnabled        = (Boolean) cs.controlPanelStates.get(9);

	    manualOverride.setSelected(manualOverrideSelected.booleanValue());
	    aggressive.setSelected(aggressiveSelected.booleanValue());
	    grid.setSelected(gridSelected.booleanValue());
	    annulus.setSelected(annulusSelected.booleanValue());
	    streamer.setSelected(streamerSelected.booleanValue());

	    manualOverride.setEnabled(manualOverrideEnabled.booleanValue());
	    aggressive.setEnabled(aggressiveEnabled.booleanValue());
	    grid.setEnabled(gridEnabled.booleanValue());
	    annulus.setEnabled(annulusEnabled.booleanValue());
	    streamer.setEnabled(streamerEnabled.booleanValue());
	}
	else {
	    manualOverride.setSelected(false);
	    aggressive.setSelected(false);
	    grid.setSelected(false);	
	    annulus.setSelected(false);
	    streamer.setSelected(false);

	    grid.setEnabled(true);	
	    enableManualOverride(false);
	    enableTrackingOptions(false);
	}
    }

    // called by the Network Client in remote app
    public void remoteSyncEvent(RemoteSyncEvent e) {
	Util.dbgPrintln("RemoteControls::remoteSyncEvent");
	setCurrentStates(e);
    }

}
