/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.reader;

import cricketdaemon.Util;
import cricketdaemon.module.*;
import cricketdaemon.logging.LoggingModule;
import java.util.*;
import java.io.*;

/**
 * Simulates a PortReader by feeding test beacon data from a file
 * generated by the Logging module.
 *
 * @author Allen Miu
 */
public class PlaybackReader extends ModuleThread
{
    BufferedReader in;  // data file stream
    long curStartTime, logStartTime, delay;
    PlaybackBeacon beacon = null;

    long breakPoint;
    boolean moreLines, realTimeMode;

    private static final int INIT_VAL = -1;

    public PlaybackReader(String inputFile, boolean rt, long breakP)
    {
	super("PlaybackReader", 0);
	moreLines = true;
	curStartTime = INIT_VAL;
	logStartTime = INIT_VAL;
	realTimeMode = rt;
	breakPoint   = breakP;
	try {
	    in = new BufferedReader(new FileReader(inputFile));
	}
	catch (IOException io) {
	    Util.assert_nonjava(false, "PlaybackReader() cannot open file: "
			+inputFile);
	}
    }

    synchronized public void run()
    {
	try {
	    while(moreLines) {
		process();
		if(realTimeMode && delay > 0) {
		    //System.out.println("wait");
		    wait(delay);
		}
		transfer(beacon.beacon, beacon.mask);
	    }
	} catch (InterruptedException e) {
	    Util.log(getName() +" interrupted");
	}
	moduleCleanUp();
	Util.log(getName() +" exits");
    }

    protected void moduleCleanUp()
    {
	try {
	    in.close();
	}
	catch (IOException e) {
	    Util.warn("PlaybackReader::moduleCleanUp() error closing file");
	}
    }

    protected void process()
    {
	String s = null;
	try {
	    s = in.readLine();
	    System.out.println("read: "+s);
	}
	catch (IOException e) {
	    Util.assert_nonjava(false, "PlaybackReader::process() error reading file");
	}

	if(s == null) {
	    moreLines = false;
	    Util.log("PlaybackReader::process() end of file");
	    return;
	}

	StringTokenizer st = new StringTokenizer(s);
	beacon = LoggingModule.parsePlayback(st);
	long beaconTimeStamp = beacon.beacon.timeStamp.longValue();
	delay = 0;

	if( beacon != null ) {
	    if(beacon.beacon.seqNum > breakPoint)
		Util.assert_nonjava(false, "Breakpoint "+breakPoint+" reached");


	    //	    System.out.println(curTimeStamp);
	    if(curStartTime == INIT_VAL) {
		curStartTime = System.currentTimeMillis();
		logStartTime = beaconTimeStamp;
	    }

	    /* send beacons */
	    beacon.beacon.timeStamp = new Long(beaconTimeStamp + curStartTime - logStartTime);

	    //System.out.println(beacon.beacon.timeStamp+" "+
	    //(System.currentTimeMillis() - beacon.beacon.timeStamp));

	    delay = (beaconTimeStamp - logStartTime) -
		(System.currentTimeMillis() - curStartTime);

	    //System.out.println("lulu: "+delay+realTimeMode);
	}


	    
    }

}
