/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.reader.parse;

import cricketdaemon.Util;

/**
 * A FieldParser that parses the input bytes as an Float.
 *
 * @author Allen Miu
 */
public class FieldParser_Float implements FieldParser
{
    public static FieldParser instance = new FieldParser_Float();

    private FieldParser_Float()
    {
    }

    // Byte order: LSB ... MSB
    /*
    public Object parse(byte[] beacon, int offset, int length)
    {
	if(length < 1 || length > 4)
	    Util.warn("FieldParser_Float: length is invalid "+length);

	int result = normalize(beacon[offset]);
	
	if(length > 1)
	    result += normalize(beacon[offset+1]) * 256;      // 2^8
	if(length > 2)
	    result += normalize(beacon[offset+2]) * 65536;    // 2^16
	if(length > 3)
	    result += normalize(beacon[offset+3]) * 16777216; // 2^24
	
	//System.out.println("FieldParser_Float "+result);

	return new Float(result);
    }
    */
    // MSB (byte 0) ... LSB 
    public Object parse(byte[] beacon, int offset, int length)
    {
	if(length < 1 || length > 4)
	    Util.warn("FieldParser_Float: length is invalid "+length);

	int result = beacon[offset];
	for(int off = 1; off < length; off++) {
	    result = result << 8;
	    result |= (0x000000ff & beacon[offset+off]);
	}	

	//System.out.println("FieldParser_Float "+result);

	return new Float(Float.intBitsToFloat(result));
    }

    // WARNING: assumes positive integer only !!!!
    public byte[] unparse(Object o)
    {
	int i = Float.floatToIntBits(((Float) o).floatValue());
	byte[] result = new byte[4];

	result[3] = (byte) (0x000000ff & i);
	result[2] = (byte) ((0x0000ff00 & i) >> 8);
	result[1] = (byte) ((0x00ff0000 & i) >> 16);
	result[0] = (byte) ((0xff000000 & i) >> 24);

	return result;
    }

    public Object stringToReturnType(String s)
    {
	Object result = null;

	try {
	    result = Float.valueOf(s);
	}
	catch(NumberFormatException e) {

	}
	return result;
    }

    public Object GPSDStringToReturnType(String s)
    {
	return stringToReturnType(s);
    }

    public static void main(String[] args) {
	FieldParser p = FieldParser_Float.instance;
	//byte[] r = p.unparse(new Float(0x12345678));
	//byte[] r = p.unparse(new Float(Float.NaN));
	byte[] r = p.unparse(new Float(-152.999456));
	for(int i = 0; i < r.length; i++)
	    System.out.println(i+" "+Integer.toHexString(r[i]));

	Float parsed = (Float) p.parse(r, 0, r.length);
	//System.out.println(Float.toHexString(parsed.intValue()));
	System.out.println(Float.toString(parsed.floatValue()));
    }
}
